/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


// Main header file
#include <ToolBox\Headers.h>


// --- Class MD5Hasher ---

const unsigned char MD5Hasher::PADDING[64] =
{
	0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

MD5Hasher::MD5Hasher()
{
	reset();
}

void MD5Hasher::finalize(unsigned char * digest)
{
	int mdi = (int) ((count1 >> 3) & 0x3F);

	unsigned int padLen = (mdi < 56) ? (56 - mdi) : (120 - mdi);
	hash(PADDING, padLen);

	unsigned long newState[16];
	for (int i=0, ii=0; i<14; i++, ii+=4)
		newState[i] = (((unsigned long) buffer[ii+3]) << 24)
			| (((unsigned long) buffer[ii+2]) << 16)
			| (((unsigned long) buffer[ii+1]) << 8)
			| ((unsigned long) buffer[ii]);

	newState[14] = count1;
	newState[15] = count2;
	transform(newState);

	for (i=0, ii=0; i<4; i++, ii+=4)
	{
		digest[ii] = (unsigned char) (state[i] & 0xFF);
		digest[ii+1] = (unsigned char) ((state[i] >> 8) & 0xFF);
		digest[ii+2] = (unsigned char) ((state[i] >> 16) & 0xFF);
		digest[ii+3] = (unsigned char) ((state[i] >> 24) & 0xFF);
	}

	reset();
}

void MD5Hasher::hash(const unsigned char * data, int length)
{
	int mdi = (int) ((count1 >> 3) & 0x3F);

	if ((count1 + ((unsigned long) length << 3)) < count1)
		count2++;
	count1 += ((unsigned long) length << 3);
	count2 += ((unsigned long) length >> 29);

	while (length--)
	{
		buffer[mdi++] = *data++;

		if (mdi == 64)
		{
			unsigned long newState[16];
			for (int i=0, ii=0; i<16; i++, ii+=4)
				newState[i] = (((unsigned long) buffer[ii+3]) << 24)
					| (((unsigned long) buffer[ii+2]) << 16)
					| (((unsigned long) buffer[ii+1]) << 8)
					| ((unsigned long) buffer[ii]);

			transform(newState);
			mdi = 0;
		}
	}
}

void MD5Hasher::reset()
{
	count1 = 0UL;
	count2 = 0UL;

	state[0] = 0x67452301UL;
	state[1] = 0xEFCDAB89UL;
	state[2] = 0x98BADCFEUL;
	state[3] = 0x10325476UL;

	for (int i=0; i<64; i++)
		buffer[i] = 0;
}
