/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_WRAPPERS_H
#define __TOOLBOX_WRAPPERS_H


namespace toolbox
{
	/**
	 * The Bool class is a wrapper for the bool type.
	 * @author Thomas Jacob
	 */
	class Bool
	{
		private:

			/**
			 * The value.
			 */
			bool Value;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new wrapper.
			 * @param value The value to be wrapped.
			 */
			Bool(bool value);

			/**
			 * Destroys the wrapper.
			 */
			~Bool();

			/**
			 * Returns the wrapper's value.
			 * @return The wrapper's value.
			 */
			inline bool Get();

			/**
			 * Sets the wrapper's value.
			 * @param value The value to be wrapped.
			 */
			inline void Set(bool value);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * The Char class is a wrapper for the char type.
	 * @author Thomas Jacob
	 */
	class Char
	{
		private:

			/**
			 * The value.
			 */
			char Value;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new wrapper.
			 * @param value The value to be wrapped.
			 */
			Char(char value);

			/**
			 * Destroys the wrapper.
			 */
			~Char();

			/**
			 * Returns the wrapper's value.
			 * @return The wrapper's value.
			 */
			inline char Get();

			/**
			 * Sets the wrapper's value.
			 * @param value The value to be wrapped.
			 */
			inline void Set(char value);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * The Double class is a wrapper for the double type.
	 * @author Thomas Jacob
	 */
	class Double
	{
		private:

			/**
			 * The value.
			 */
			double Value;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new wrapper.
			 * @param value The value to be wrapped.
			 */
			Double(double value);

			/**
			 * Destroys the wrapper.
			 */
			~Double();

			/**
			 * Returns the wrapper's value.
			 * @return The wrapper's value.
			 */
			inline double Get();

			/**
			 * Sets the wrapper's value.
			 * @param value The value to be wrapped.
			 */
			inline void Set(double value);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * The Float class is a wrapper for the float type.
	 * @author Thomas Jacob
	 */
	class Float
	{
		private:

			/**
			 * The value.
			 */
			float Value;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new wrapper.
			 * @param value The value to be wrapped.
			 */
			Float(float value);

			/**
			 * Destroys the wrapper.
			 */
			~Float();

			/**
			 * Returns the wrapper's value.
			 * @return The wrapper's value.
			 */
			inline float Get();

			/**
			 * Sets the wrapper's value.
			 * @param value The value to be wrapped.
			 */
			inline void Set(float value);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * The Int class is a wrapper for the int type.
	 * @author Thomas Jacob
	 */
	class Int
	{
		private:

			/**
			 * The value.
			 */
			int Value;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new wrapper.
			 * @param value The value to be wrapped.
			 */
			Int(int value);

			/**
			 * Destroys the wrapper.
			 */
			~Int();

			/**
			 * Returns the wrapper's value.
			 * @return The wrapper's value.
			 */
			inline int Get();

			/**
			 * Sets the wrapper's value.
			 * @param value The value to be wrapped.
			 */
			inline void Set(int value);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * The LongDouble class is a wrapper for the long double type.
	 * @author Thomas Jacob
	 */
	class LongDouble
	{
		private:

			/**
			 * The value.
			 */
			long double Value;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new wrapper.
			 * @param value The value to be wrapped.
			 */
			LongDouble(long double value);

			/**
			 * Destroys the wrapper.
			 */
			~LongDouble();

			/**
			 * Returns the wrapper's value.
			 * @return The wrapper's value.
			 */
			inline long double Get();

			/**
			 * Sets the wrapper's value.
			 * @param value The value to be wrapped.
			 */
			inline void Set(long double value);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * The LongInt class is a wrapper for the long type.
	 * @author Thomas Jacob
	 */
	class LongInt
	{
		private:

			/**
			 * The value.
			 */
			long Value;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new wrapper.
			 * @param value The value to be wrapped.
			 */
			LongInt(long value);

			/**
			 * Destroys the wrapper.
			 */
			~LongInt();

			/**
			 * Returns the wrapper's value.
			 * @return The wrapper's value.
			 */
			inline long Get();

			/**
			 * Sets the wrapper's value.
			 * @param value The value to be wrapped.
			 */
			inline void Set(long value);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * The Pointer class is a wrapper for the void * type.
	 * @author Thomas Jacob
	 */
	class Pointer
	{
		private:

			/**
			 * The value.
			 */
			void * Value;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new wrapper.
			 * @param value The value to be wrapped.
			 */
			Pointer(void * value);

			/**
			 * Destroys the wrapper.
			 */
			~Pointer();

			/**
			 * Returns the wrapper's value.
			 * @return The wrapper's value.
			 */
			inline void * Get();

			/**
			 * Sets the wrapper's value.
			 * @param value The value to be wrapped.
			 */
			inline void Set(void * value);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * The ShortInt class is a wrapper for the short type.
	 * @author Thomas Jacob
	 */
	class ShortInt
	{
		private:

			/**
			 * The value.
			 */
			short Value;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new wrapper.
			 * @param value The value to be wrapped.
			 */
			ShortInt(short value);

			/**
			 * Destroys the wrapper.
			 */
			~ShortInt();

			/**
			 * Returns the wrapper's value.
			 * @return The wrapper's value.
			 */
			inline short Get();

			/**
			 * Sets the wrapper's value.
			 * @param value The value to be wrapped.
			 */
			inline void Set(short value);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * The SignedChar class is a wrapper for the signed char type.
	 * @author Thomas Jacob
	 */
	class SignedChar
	{
		private:

			/**
			 * The value.
			 */
			signed char Value;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new wrapper.
			 * @param value The value to be wrapped.
			 */
			SignedChar(signed char value);

			/**
			 * Destroys the wrapper.
			 */
			~SignedChar();

			/**
			 * Returns the wrapper's value.
			 * @return The wrapper's value.
			 */
			inline signed char Get();

			/**
			 * Sets the wrapper's value.
			 * @param value The value to be wrapped.
			 */
			inline void Set(signed char value);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * The SignedInt class is a wrapper for the signed int type.
	 * @author Thomas Jacob
	 */
	class SignedInt
	{
		private:

			/**
			 * The value.
			 */
			signed int Value;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new wrapper.
			 * @param value The value to be wrapped.
			 */
			SignedInt(signed int value);

			/**
			 * Destroys the wrapper.
			 */
			~SignedInt();

			/**
			 * Returns the wrapper's value.
			 * @return The wrapper's value.
			 */
			inline signed int Get();

			/**
			 * Sets the wrapper's value.
			 * @param value The value to be wrapped.
			 */
			inline void Set(signed int value);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * The SignedLongInt class is a wrapper for the signed long type.
	 * @author Thomas Jacob
	 */
	class SignedLongInt
	{
		private:

			/**
			 * The value.
			 */
			signed long Value;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new wrapper.
			 * @param value The value to be wrapped.
			 */
			SignedLongInt(signed long value);

			/**
			 * Destroys the wrapper.
			 */
			~SignedLongInt();

			/**
			 * Returns the wrapper's value.
			 * @return The wrapper's value.
			 */
			inline signed long Get();

			/**
			 * Sets the wrapper's value.
			 * @param value The value to be wrapped.
			 */
			inline void Set(signed long value);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * The SignedShortInt class is a wrapper for the signed short type.
	 * @author Thomas Jacob
	 */
	class SignedShortInt
	{
		private:

			/**
			 * The value.
			 */
			signed short Value;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new wrapper.
			 * @param value The value to be wrapped.
			 */
			SignedShortInt(signed short value);

			/**
			 * Destroys the wrapper.
			 */
			~SignedShortInt();

			/**
			 * Returns the wrapper's value.
			 * @return The wrapper's value.
			 */
			inline signed short Get();

			/**
			 * Sets the wrapper's value.
			 * @param value The value to be wrapped.
			 */
			inline void Set(signed short value);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * The UnsignedChar class is a wrapper for the unsigned char type.
	 * @author Thomas Jacob
	 */
	class UnsignedChar
	{
		private:

			/**
			 * The value.
			 */
			unsigned char Value;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new wrapper.
			 * @param value The value to be wrapped.
			 */
			UnsignedChar(unsigned char value);

			/**
			 * Destroys the wrapper.
			 */
			~UnsignedChar();

			/**
			 * Returns the wrapper's value.
			 * @return The wrapper's value.
			 */
			inline unsigned char Get();

			/**
			 * Sets the wrapper's value.
			 * @param value The value to be wrapped.
			 */
			inline void Set(unsigned char value);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * The UnsignedInt class is a wrapper for the unsigned int type.
	 * @author Thomas Jacob
	 */
	class UnsignedInt
	{
		private:

			/**
			 * The value.
			 */
			unsigned int Value;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new wrapper.
			 * @param value The value to be wrapped.
			 */
			UnsignedInt(unsigned int value);

			/**
			 * Destroys the wrapper.
			 */
			~UnsignedInt();

			/**
			 * Returns the wrapper's value.
			 * @return The wrapper's value.
			 */
			inline unsigned int Get();

			/**
			 * Sets the wrapper's value.
			 * @param value The value to be wrapped.
			 */
			inline void Set(unsigned int value);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * The UnsignedLongInt class is a wrapper for the unsigned long type.
	 * @author Thomas Jacob
	 */
	class UnsignedLongInt
	{
		private:

			/**
			 * The value.
			 */
			unsigned long Value;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new wrapper.
			 * @param value The value to be wrapped.
			 */
			UnsignedLongInt(unsigned long value);

			/**
			 * Destroys the wrapper.
			 */
			~UnsignedLongInt();

			/**
			 * Returns the wrapper's value.
			 * @return The wrapper's value.
			 */
			inline unsigned long Get();

			/**
			 * Sets the wrapper's value.
			 * @param value The value to be wrapped.
			 */
			inline void Set(unsigned long value);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * The UnsignedShortInt class is a wrapper for the unsigned short type.
	 * @author Thomas Jacob
	 */
	class UnsignedShortInt
	{
		private:

			/**
			 * The value.
			 */
			unsigned short Value;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new wrapper.
			 * @param value The value to be wrapped.
			 */
			UnsignedShortInt(unsigned short value);

			/**
			 * Destroys the wrapper.
			 */
			~UnsignedShortInt();

			/**
			 * Returns the wrapper's value.
			 * @return The wrapper's value.
			 */
			inline unsigned short Get();

			/**
			 * Sets the wrapper's value.
			 * @param value The value to be wrapped.
			 */
			inline void Set(unsigned short value);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * The Void class is a wrapper for the void type.
	 * It contains no data.
	 * @author Thomas Jacob
	 */
	class Void
	{
		private:

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new wrapper.
			 */
			Void();

			/**
			 * Destroys the wrapper.
			 */
			~Void();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};
}


#endif
